@@ -1,5 +1,5 @@
-require 'puppet/indirector/store_configs'
-require 'puppet/resource/catalog'
+require_relative '../../../puppet/indirector/store_configs'
+require_relative '../../../puppet/resource/catalog'
 
 class Puppet::Resource::Catalog::StoreConfigs < Puppet::Indirector::StoreConfigs
 
